/*
 * linux/arch/arm/mach-at91/board-panelcard.c
 *
 *  Copyright (C) 2007 taskit
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/types.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/spi/spi.h>
#include <linux/spi/ads7846.h>
#include <linux/dm9000.h>
#include <linux/fb.h>
#include <linux/mtd/physmap.h>
#include <linux/portuxinput.h>
#include <linux/at91-matrix.h>
#include <linux/gpio_keys.h>

#include <video/atmel_lcdc.h>
#include <video/panelcard_bl.h>

#include <asm/delay.h>
#include <asm/hardware.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/irq.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <asm/arch/board.h>
#include <asm/arch/gpio.h>
#include <asm/arch/at91sam926x_mc.h>
#include <asm/arch/at91_rtt.h>

#include "generic.h"


/*
 * Serial port configuration.
 *    0 .. 2 = USART0 .. USART2
 *    3      = DBGU
 */
static struct at91_uart_config __initdata panelcard_uart_config = {
	.console_tty	= 3,				/* dbgu */
	.nr_tty		= 4,
	.tty_map	= { 3, -1, 1, -1 }		/* ttyS0, ..., ttyS3 */
};

static void __init panelcard_map_io(void)
{
	/* Initialize processor: 18.432 MHz crystal */
	at91sam9261_initialize(18432000);

	/* Setup the serial ports and console */
	at91_init_serial(&panelcard_uart_config);
}

static void __init panelcard_init_irq(void)
{
	at91sam9261_init_interrupts(NULL);
}


/*
 * DM9000 ethernet device
 */
#if defined(CONFIG_DM9000)
static struct resource at91sam9261_dm9000_resource[] = {
	[0] = {
		.start	= AT91_CHIPSELECT_2,
		.end	= AT91_CHIPSELECT_2 + 3,
		.flags	= IORESOURCE_MEM
	},
	[1] = {
		.start	= AT91_CHIPSELECT_2 + 0x44,
		.end	= AT91_CHIPSELECT_2 + 0xFF,
		.flags	= IORESOURCE_MEM
	},
	[2] = {
		.start	= AT91_PIN_PC11,
		.end	= AT91_PIN_PC11,
		.flags	= IORESOURCE_IRQ
	}
};

static struct dm9000_plat_data dm9000_platdata = {
	.flags		= DM9000_PLATF_16BITONLY,
};

static struct platform_device at91sam9261_dm9000_device = {
	.name		= "dm9000",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(at91sam9261_dm9000_resource),
	.resource	= at91sam9261_dm9000_resource,
	.dev		= {
		.platform_data	= &dm9000_platdata,
	}
};

static void __init panelcard_add_device_dm9000(void)
{
	/*
	 * Configure Chip-Select 2 on SMC for the DM9000.
	 * Note: These timings were calculated for MASTER_CLOCK = 100000000
	 *  according to the DM9000 timings.
	 */
	at91_sys_write(AT91_SMC_SETUP(2), AT91_SMC_NWESETUP_(2) | AT91_SMC_NCS_WRSETUP_(0) | AT91_SMC_NRDSETUP_(2) | AT91_SMC_NCS_RDSETUP_(0));
	at91_sys_write(AT91_SMC_PULSE(2), AT91_SMC_NWEPULSE_(4) | AT91_SMC_NCS_WRPULSE_(8) | AT91_SMC_NRDPULSE_(4) | AT91_SMC_NCS_RDPULSE_(8));
	at91_sys_write(AT91_SMC_CYCLE(2), AT91_SMC_NWECYCLE_(16) | AT91_SMC_NRDCYCLE_(16));
	at91_sys_write(AT91_SMC_MODE(2), AT91_SMC_READMODE | AT91_SMC_WRITEMODE | AT91_SMC_EXNWMODE_DISABLE | AT91_SMC_BAT_WRITE | AT91_SMC_DBW_16 | AT91_SMC_TDF_(1));

	/* Configure Reset signal as output */
	at91_set_gpio_output(AT91_PIN_PC10, 0);

	/* Configure Interrupt pin as input, no pull-up */
	at91_set_gpio_input(AT91_PIN_PC11, 0);

	platform_device_register(&at91sam9261_dm9000_device);
}
#else
static void __init panelcard_add_device_dm9000(void) {}
#endif /* CONFIG_DM9000 */


/*
 * USB Host Port
 */
static struct at91_usbh_data __initdata panelcard_usbh_data = {
	.ports		= 2,
};


/*
 * USB Device Port
 */
static struct at91_udc_data __initdata panelcard_udc_data = {
	.vbus_pin	= AT91_PIN_PC2,
	.pullup_pin	= 0,		/* pull-up driven by UDC */
};


/*
 * MCI (SD/MMC)
 */
static struct at91_mmc_data __initdata panelcard_mmc_data = {
	.wire4		= 1,
//	.det_pin	= ... not connected
//	.wp_pin		= ... not connected
//	.vcc_pin	= ... not connected
};

/*
 * LCD Controller
 */
#if !defined(CONFIG_PANELCARD_NODISPLAY) && (defined(CONFIG_FB_ATMEL) || defined(CONFIG_FB_ATMEL_MODULE))

#define AT91SAM9261_DEFAULT_LCDCON2 	(ATMEL_LCDC_MEMOR_LITTLE \
					| ATMEL_LCDC_DISTYPE_TFT    \
					| ATMEL_LCDC_CLKMOD_ALWAYSACTIVE \
					| ATMEL_LCDC_INVCLK_INVERTED)

static void at91_lcdc_power_control(int on)
{
	int i;
	if (on) {
		for(i = 0; i < 10; i++) {
			at91_set_gpio_value(AT91_PIN_PC10, 0);	/* power up */
			at91_set_gpio_value(AT91_PIN_PC10, 1);	/* power down */
			if(i < 4)
				udelay(1);
		}
		at91_set_gpio_value(AT91_PIN_PC10, 0);	/* power up */
	} else
		at91_set_gpio_value(AT91_PIN_PC10, 1);	/* power down */
}

#if defined(CONFIG_PANELCARD_EDT35) || defined(CONFIG_PANELCARD_EDT57QVGA)
static struct fb_videomode at91_tft_vga_modes[] = {
	{
		.name           = "EDT035005DM6 @ 60",
		.refresh	= 60,
		.xres		= 320,		.yres		= 240,
		.pixclock	= KHZ2PICOS(6400),

		.left_margin	= 38,
		.right_margin	= 20,
		.upper_margin	= 15,
		.lower_margin	= 5,
		.hsync_len	= 30,
		.vsync_len	= 5,

		.sync		= 0,
		.vmode		= FB_VMODE_NONINTERLACED,
	},
};

static struct fb_monspecs at91fb_default_monspecs = {
	.manufacturer	= "EDT",
	.monitor        = "ET035005DM6",

	.modedb		= at91_tft_vga_modes,
	.modedb_len	= ARRAY_SIZE(at91_tft_vga_modes),
	.hfmin		= 15000,
	.hfmax		= 64000,
	.vfmin		= 50,
	.vfmax		= 150,
};

static struct atmel_lcdfb_info __initdata panelcard_lcdc_data = {
	.default_bpp			= CONFIG_AT91_DEFAULT_LCD_DEPTH,
#if defined(CONFIG_PANELCARD_EDT57QVGA)
	.default_cval			= 0xff,
#else
	.default_cval			= 0x80,
#endif
	.default_dmacon			= ATMEL_LCDC_DMAEN,
	.default_lcdcon2		= AT91SAM9261_DEFAULT_LCDCON2,
	.default_monspecs		= &at91fb_default_monspecs,
	.atmel_lcdfb_power_control	= at91_lcdc_power_control,
	.guard_time			= 1,
#if  defined(CONFIG_PANELCARD_EDT57QVGA)
	.lcd_wiring_mode                = ATMEL_LCDC_WIRING_RGB555,
#endif
};

static struct panelcard_bl_data panelcard_bl_data = {
#if defined(CONFIG_PANELCARD_EDT57QVGA)
	.lcdcon_is_backlight = 1,
	.lcdcon_inverted = 0,
	.max_lcdcon = 0xff
#endif
};
#elif defined(CONFIG_PANELCARD_EDT57VGA)
static struct fb_videomode at91_tft_vga_modes[] = {
	{
		.name           = "ET057009DHU @ 60",
		.refresh	= 60,
		.xres		= 640,		.yres		= 480,
		.pixclock	= KHZ2PICOS(25175),

		.left_margin	= 144,
		.right_margin	= 16,
		.upper_margin	= 35,
		.lower_margin	= 10,
		.hsync_len	= 30,
		.vsync_len	= 5,

		.sync		= 0,
		.vmode		= FB_VMODE_NONINTERLACED,
	},
};

static struct fb_monspecs at91fb_default_monspecs = {
	.manufacturer	= "EDT",
	.monitor        = "ET057009DHU",

	.modedb		= at91_tft_vga_modes,
	.modedb_len	= ARRAY_SIZE(at91_tft_vga_modes),
	.hfmin		= 15000,
	.hfmax		= 64000,
	.vfmin		= 50,
	.vfmax		= 150,
};

static struct atmel_lcdfb_info __initdata panelcard_lcdc_data = {
	.default_bpp			= CONFIG_AT91_DEFAULT_LCD_DEPTH,
	.default_cval			= 0x00,
	.default_burst_length		= 2,
	.default_dmacon			= ATMEL_LCDC_DMAEN,
	.default_lcdcon2		= AT91SAM9261_DEFAULT_LCDCON2,
	.default_monspecs		= &at91fb_default_monspecs,
	.atmel_lcdfb_power_control	= at91_lcdc_power_control,
	.guard_time			= 1,
	.lcd_wiring_mode                = ATMEL_LCDC_WIRING_RGB555,
};

static struct panelcard_bl_data panelcard_bl_data = {
	.lcdcon_is_backlight = 1,
	.lcdcon_inverted = 1,
	.max_lcdcon = 0xff
};
#elif defined(CONFIG_PANELCARD_EDT70)
static struct fb_videomode at91_tft_vga_modes[] = {
	{
		.name           = "EDT035005DM6 @ 60",
		.refresh	= 40,
		.xres		= 800,		.yres		= 480,
		.pixclock	= KHZ2PICOS(28000),

		.left_margin	= 40,
		.right_margin	= 40,
		.upper_margin	= 3,
		.lower_margin	= 0,
		.hsync_len	= 48,
		.vsync_len	= 3,
		.sync		= 0,
		.vmode		= FB_VMODE_NONINTERLACED,
	},
};

static struct fb_monspecs at91fb_default_monspecs = {
	.manufacturer	= "EDT",
	.monitor        = "ET035005DM6",

	.modedb		= at91_tft_vga_modes,
	.modedb_len	= ARRAY_SIZE(at91_tft_vga_modes),
	.hfmin		= 10000,
	.hfmax		= 44000,
	.vfmin		= 30,
	.vfmax		= 80,
};

static struct atmel_lcdfb_info __initdata panelcard_lcdc_data = {
	.default_bpp			= CONFIG_AT91_DEFAULT_LCD_DEPTH,
	.default_cval			= 0x3f,
	.default_burst_length		= 2,
	.default_dmacon			= ATMEL_LCDC_DMAEN,
	.default_lcdcon2		= AT91SAM9261_DEFAULT_LCDCON2,
	.default_monspecs		= &at91fb_default_monspecs,
	.atmel_lcdfb_power_control	= at91_lcdc_power_control,
	.guard_time			= 1,
	.lcd_wiring_mode		= ATMEL_LCDC_WIRING_RGB555,
};

static struct panelcard_bl_data panelcard_bl_data = {
	.lcdcon_is_backlight = 1,
	.lcdcon_inverted = 1,
	.max_lcdcon = 0xc0
};
#elif defined(CONFIG_PANELCARD_SHARP)
static struct fb_videomode at91_tft_vga_modes[] = {
	{
		.name           = "LQ035Q7DH06 @ 60",
		.refresh	= 60,
		.xres		= 240,		.yres		= 320,
		.pixclock	= KHZ2PICOS(4965),

		.left_margin	= 8,
		.right_margin	= 33,
		.upper_margin	= 8,
		.lower_margin	= 0,
		.hsync_len	= 5,
		.vsync_len	= 1,

		.sync		= 0,
		.vmode		= FB_VMODE_NONINTERLACED,
	},
};

static struct fb_monspecs at91fb_default_monspecs = {
	.manufacturer	= "SHA",
	.monitor        = "LQ035Q7DH06",

	.modedb		= at91_tft_vga_modes,
	.modedb_len	= ARRAY_SIZE(at91_tft_vga_modes),
	.hfmin		= 15000,
	.hfmax		= 64000,
	.vfmin		= 50,
	.vfmax		= 150,
};

static struct atmel_lcdfb_info __initdata panelcard_lcdc_data = {
	.default_bpp			= CONFIG_AT91_DEFAULT_LCD_DEPTH,
	.default_dmacon			= ATMEL_LCDC_DMAEN,
	.default_lcdcon2		= AT91SAM9261_DEFAULT_LCDCON2,
	.default_monspecs		= &at91fb_default_monspecs,
	.atmel_lcdfb_power_control	= at91_lcdc_power_control,
	.guard_time			= 1,
};

static struct panelcard_bl_data panelcard_bl_data;
#endif

static struct platform_device panelcard_bl_device = {
	.name = "panelcard-bl",
	.id = -1,
	.dev = {
		.platform_data = &panelcard_bl_data,
	},
};

static void __init panelcard_register_backlight(void) {
	platform_device_register(&panelcard_bl_device);
}

#else
static struct atmel_lcdfb_info __initdata panelcard_lcdc_data;
static void __init panelcard_register_backlight(void) {}
#endif

#define PANELCARD_FLASH_BASE	AT91_CHIPSELECT_0
#define PANELCARD_FLASH_SIZE	0x4000000

static struct physmap_flash_data panelcard_flash_data = {
	.width		= 2,
};

static struct resource panelcard_flash_resource = {
	.start		= PANELCARD_FLASH_BASE,
	.end		= PANELCARD_FLASH_BASE + PANELCARD_FLASH_SIZE - 1,
	.flags		= IORESOURCE_MEM,
};

static struct platform_device panelcard_flash = {
	.name		= "physmap-flash",
	.id		= 0,
	.dev		= {
				.platform_data	= &panelcard_flash_data,
			},
	.resource	= &panelcard_flash_resource,
	.num_resources	= 1,
};

#if !defined(CONFIG_PANELCARD_NODISPLAY) && (defined(CONFIG_TOUCHSCREEN_ADS7846) || defined(CONFIG_TOUCHSCREEN_ADS7846_MODULE))
static int ads7843_pendown_state(void)
{
        return !at91_get_gpio_value(AT91_PIN_PA25);      /* Touchscreen PENIRQ */
}

static void __init ek_add_device_ts(void)
{
        at91_set_gpio_input(AT91_PIN_PA25, 1);  /* External IRQ0, with pullup */
        at91_set_gpio_input(AT91_PIN_PA26, 1);  /* Touchscreen BUSY signal */
}

#if defined(CONFIG_PANELCARD_EDT35)
static struct ads7846_platform_data ads_info = {
	.model                  = 7843,
	.x_min                  = 190,
	.x_max                  = 3880,
	.y_min                  = 190,
	.y_max                  = 3880,
	.x_inverted             = 1,
	.y_inverted             = 1,
	.vref_delay_usecs       = 100,
	.x_plate_ohms           = 1000,
	.y_plate_ohms           = 1000,
	.pressure_max           = 0,
	.debounce_max           = 1,
	.debounce_rep           = 5,
	.debounce_tol           = 8,
	.get_pendown_state      = ads7843_pendown_state,
};
#elif defined(CONFIG_PANELCARD_EDT57QVGA)
static struct ads7846_platform_data ads_info = {
	.model                  = 7846,
	.x_min                  = 190,
	.x_max                  = 3880,
	.y_min                  = 190,
	.y_max                  = 3880,
	.x_inverted             = 1,
	.y_inverted             = 1,
	.vref_delay_usecs       = 100,
	.settle_delay_usecs     = 100,
	.penirq_recheck_delay_usecs = 2000,
	.x_plate_ohms           = 800,
	.y_plate_ohms           = 450,
	.pressure_max           = 1000,
	.debounce_max           = 1,
	.debounce_rep           = 3,
	.debounce_tol           = 30,
	.get_pendown_state      = ads7843_pendown_state,
};
#elif defined(CONFIG_PANELCARD_EDT57VGA)
static struct ads7846_platform_data ads_info = {
	.model                  = 7846,
	.x_min                  = 190,
	.x_max                  = 3880,
	.y_min                  = 190,
	.y_max                  = 3880,
	.x_inverted             = 1,
	.y_inverted             = 1,
	.vref_delay_usecs       = 100,
	.settle_delay_usecs     = 100,
	.penirq_recheck_delay_usecs = 2000,
	.x_plate_ohms           = 800,
	.y_plate_ohms           = 450,
	.pressure_max           = 1000,
	.debounce_max           = 1,
	.debounce_rep           = 3,
	.debounce_tol           = 30,
	.get_pendown_state      = ads7843_pendown_state,
};
#elif defined(CONFIG_PANELCARD_EDT70)
static struct ads7846_platform_data ads_info = {
	.model                  = 7846,
	.x_min                  = 130,
	.x_max                  = 3850,
	.y_min                  = 270,
	.y_max                  = 3820,
	.x_inverted             = 1,
	.y_inverted             = 1,
	.vref_delay_usecs       = 100,
	.settle_delay_usecs     = 100,
	.penirq_recheck_delay_usecs = 2000,
	.x_plate_ohms           = 700,
	.y_plate_ohms           = 400,
	.pressure_max           = 15000,
	.debounce_max           = 1,
	.debounce_rep           = 3,
	.debounce_tol           = 60,
	.get_pendown_state      = ads7843_pendown_state,
};
#elif defined(CONFIG_PANELCARD_SHARP)
static struct ads7846_platform_data ads_info = {
	.model                  = 7843,
	.x_min                  = 240,
	.x_max                  = 3750,
	.y_min                  = 200,
	.y_max                  = 3850,
	.x_inverted             = 0,
	.y_inverted             = 1,
	.vref_delay_usecs       = 100,
	.x_plate_ohms           = 1000,
	.y_plate_ohms           = 1000,
	.pressure_max           = 0,
	.debounce_max           = 1,
	.debounce_rep           = 5,
	.debounce_tol           = 8,
	.get_pendown_state      = ads7843_pendown_state,
};
#endif

static struct spi_board_info panelcard_spi_devices[] = {
	{ /* User accessible spi - cs1 */
		.modalias = "ads7846",
		.chip_select  = 1,
		.max_speed_hz = 125000 * 4,
		.bus_num = 1,
		.irq = AT91_PIN_PA25,
		.controller_data = AT91_PIN_PA24,
		.platform_data = &ads_info,
	},
};
#else
static void __init ek_add_device_ts(void) {}
#endif

#if defined(CONFIG_RTC_DRV_AT91SAM926X) || defined(CONFIG_RTC_DRV_AT91SAM926X_MODULE)

static struct at91_rtt_data rtt_data={0};

static struct platform_device at91sam926x_rtc_device = {
	.name		= "at91_rtt",
	.id		= -1,
        .dev		= {
        	.platform_data	= &rtt_data,
	},
	.num_resources	= 0,
};

void __init at91_add_device_rtc(void)
{
	platform_device_register(&at91sam926x_rtc_device);
}
#else
void __init at91_add_device_rtc(void) {}
#endif

#if defined(CONFIG_INPUT_PORTUXMATRIX) || defined(CONFIG_INPUT_PORTUXMATRIX_MODULE)
static keyarray portuxinput_keyarraymap={
	{{0,0},{0,0},{0,KEY_MINUS},{0,KEY_SLASH},{0,KEY_BACKSLASH},{0,KEY_EQUAL},{0,0},{0,0},
	 {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,KEY_DOT},{0,KEY_SEMICOLON},{0,KEY_COMMA},{0,KEY_MINUS},{0,KEY_SLASH},{0,KEY_BACKSLASH},{0,KEY_EQUAL},{0,KEY_LEFTBRACE},
	 {0,KEY_RIGHTBRACE},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,KEY_A},{0,KEY_B},{0,KEY_C},{0,0},{0,0},{0,0},{0,0},{0,0},
	 {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,KEY_D},{0,KEY_E},{0,KEY_F},{0,0},{0,0},{0,0},{0,0},{0,0},
	 {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,KEY_G},{0,KEY_H},{0,KEY_I},{0,0},{0,0},{0,0},{0,0},{0,0},
	 {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,KEY_J},{0,KEY_K},{0,KEY_L},{0,0},{0,0},{0,0},{0,0},{0,0},
	 {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,KEY_M},{0,KEY_N},{0,KEY_O},{0,0},{0,0},{0,0},{0,0},{0,0},
	 {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,KEY_P},{0,KEY_Q},{0,KEY_R},{0,KEY_S},{0,0},{0,0},{0,0},{0,0},
	 {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,KEY_T},{0,KEY_U},{0,KEY_V},{0,0},{0,0},{0,0},{0,0},{0,0},
	 {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,KEY_W},{0,KEY_X},{0,KEY_Y},{0,KEY_Z},{0,0},{0,0},{0,0},{0,0},
	 {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	};

static matrixmap portuxinput_matrixmap={{
	{{PORTUXINPUT_MASK_SETKEYMAP,1},{0,KEY_LEFT},{0,KEY_ENTER},{0,KEY_1},{0,KEY_4},{0,KEY_7},{0,KEY_KPASTERISK},{0,0}},
	{{0,KEY_UP},{0,KEY_TAB},{0,KEY_DOWN},{0,KEY_2},{0,KEY_5},{0,KEY_8},{0,KEY_0},{0,0}},
	{{0,KEY_BACKSPACE},{0,KEY_RIGHT},{0,KEY_ESC},{0,KEY_3},{0,KEY_6},{0,KEY_9},{PORTUXINPUT_MASK_LSHIFT,KEY_3},{0,0}},
	{{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}}
},{
	{{PORTUXINPUT_MASK_SETKEYMAP,0},{0x82,-4},{0,KEY_ENTER},{PORTUXINPUT_MASK_KEYARRAY,1},{PORTUXINPUT_MASK_KEYARRAY,4},{PORTUXINPUT_MASK_KEYARRAY,7},{0,KEY_KPPLUS},{0,0}},
	{{0x81,4},{0,BTN_MOUSE},{0x81,-4},{PORTUXINPUT_MASK_KEYARRAY,2},{PORTUXINPUT_MASK_KEYARRAY,5},{PORTUXINPUT_MASK_KEYARRAY,8},{0,KEY_SPACE},{0,0}},
	{{0,KEY_BACKSPACE},{0x82,4},{0,KEY_ESC},{PORTUXINPUT_MASK_KEYARRAY,3},{PORTUXINPUT_MASK_KEYARRAY,6},{PORTUXINPUT_MASK_KEYARRAY,9},{0,KEY_CAPSLOCK},{0,0}},
	{{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},
	{{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}}
}};
#endif

#if defined(CONFIG_INPUT_PORTUXINPUT) || defined(CONFIG_INPUT_PORTUXINPUT_MODULE)
static struct portuxinput_data panelcard_portuxinput_data = {
	.irq = AT91_PIN_PA27,
#if defined(CONFIG_INPUT_PORTUXMATRIX) || defined(CONFIG_INPUT_PORTUXMATRIX_MODULE)
	.keyarraymap = &portuxinput_keyarraymap,
	.matrixkeymap = &portuxinput_matrixmap,
#endif
};

static struct platform_device panelcard_portuxinput_device = {
	.name	= PORTUXINPUT_DEVNAME,
	.id	= -1,
	.dev = {
		.platform_data = &panelcard_portuxinput_data,
	},
	.num_resources	= 0,
};
#endif

#if defined(CONFIG_INPUT_AT91_MATRIX) || defined(CONFIG_INPUT_AT91_MATRIX_MODULE)
struct at91_matrix_data panelcard_at91_matrix_data = {
	.col    = {AT91_PIN_PC14, AT91_PIN_PC9, AT91_PIN_PC8, 0, 0, 0, 0, 0},
	.row    = {AT91_PIN_PA11, AT91_PIN_PA14, AT91_PIN_PA15, AT91_PIN_PA16, 0, 0, 0, 0},
	.keymap = {{KEY_1, KEY_2, KEY_3, 0, 0, 0, 0, 0},
	           {KEY_4, KEY_5, KEY_6, 0, 0, 0, 0, 0},
	           {KEY_7, KEY_8, KEY_9, 0, 0, 0, 0, 0},
	           {KEY_KPASTERISK, KEY_0,KEY_ENTER, 0, 0, 0, 0, 0},
	           {0, 0, 0, 0, 0, 0, 0, 0},
	           {0, 0, 0, 0, 0, 0, 0, 0},
	           {0, 0, 0, 0, 0, 0, 0, 0},
	           {0, 0, 0, 0, 0, 0, 0, 0}}
};

static struct platform_device panelcard_at91_matrix_device = {
	.name  = "at91_matrix",
	.id    = -1,
	.dev   = {
		.platform_data = &panelcard_at91_matrix_data,
	},
	.num_resources  = 0,
};
#endif


#if (defined(CONFIG_PANELCARD_EDT57QVGA) || defined(CONFIG_PANELCARD_EDT57VGA) || defined(CONFIG_PANELCARD_EDT70)) && \
    (defined(CONFIG_KEYBOARD_GPIO) || defined(CONFIG_KEYBOARD_GPIO_MODULE))
static struct gpio_keys_button panelcard_buttons[] = {
	{
		.gpio		= AT91_PIN_PA17,
		.code		= KEY_LEFT,
		.desc		= "Left",
		.active_low	= 1,
	},
	{
		.gpio		= AT91_PIN_PA18,
		.code		= KEY_UP,
		.desc		= "Up",
		.active_low	= 1,
	},
	{
		.gpio		= AT91_PIN_PA19,
		.code		= KEY_ENTER,
		.desc		= "Enter",
		.active_low	= 1,
	},
	{
		.gpio		= AT91_PIN_PA20,
		.code		= KEY_DOWN,
		.desc		= "Down",
		.active_low	= 1,
	},
	{
		.gpio		= AT91_PIN_PA21,
		.code		= KEY_RIGHT,
		.desc		= "Right",
		.active_low	= 1,
	}
};

static struct gpio_keys_platform_data panelcard_button_data = {
	.buttons	= panelcard_buttons,
	.nbuttons	= ARRAY_SIZE(panelcard_buttons),
};

static struct platform_device panelcard_button_device = {
	.name		= "gpio-keys",
	.id		= -1,
	.num_resources	= 0,
	.dev		= {
		.platform_data	= &panelcard_button_data,
	}
};

static void __init panelcard_add_device_buttons(void)
{
	at91_set_gpio_input(AT91_PIN_PA17, 1);	/* Left */
	at91_set_deglitch(AT91_PIN_PA17, 1);
	at91_set_gpio_input(AT91_PIN_PA18, 1);	/* Up */
	at91_set_deglitch(AT91_PIN_PA18, 1);
	at91_set_gpio_input(AT91_PIN_PA19, 1);	/* Enter */
	at91_set_deglitch(AT91_PIN_PA19, 1);
	at91_set_gpio_input(AT91_PIN_PA20, 1);	/* Down */
	at91_set_deglitch(AT91_PIN_PA20, 1);
	at91_set_gpio_input(AT91_PIN_PA21, 1);	/* Right */
	at91_set_deglitch(AT91_PIN_PA21, 1);

	platform_device_register(&panelcard_button_device);
}
#else
static void __init panelcard_add_device_buttons(void) {}
#endif

#ifdef CONFIG_W1_MASTER_AT91GPIO
static struct at91_gpio_w1_data panelcard_w1_data = {
	.pin = AT91_PIN_PA3,
};

static struct platform_device panelcard_w1_device = {
	.name	= "w1_at91gpio",
	.id	= -1,
	.dev = {
		.platform_data = &panelcard_w1_data,
	},
	.num_resources	= 0,
};

static void __init panelcard_add_w1(void)
{
	platform_device_register(&panelcard_w1_device);
}
#else
static void __init panelcard_add_w1(void) {}
#endif

static void __init panelcard_board_init(void)
{
	/* Serial */
	at91_add_device_serial();
	/* USB Host */
	at91_add_device_usbh(&panelcard_usbh_data);
	/* USB Device */
	at91_add_device_udc(&panelcard_udc_data);
	/* I2C */
	at91_add_device_i2c(NULL, 0);
	/* Touchscreen */
	ek_add_device_ts();
	/* SPI */
#if !defined(CONFIG_PANELCARD_NODISPLAY) && (defined(CONFIG_TOUCHSCREEN_ADS7846) || defined(CONFIG_TOUCHSCREEN_ADS7846_MODULE))
	at91_add_device_spi(panelcard_spi_devices, 1);
#endif
	/* DM9000 ethernet */
	panelcard_add_device_dm9000();
	/* MMC */
	at91_add_device_mmc(0, &panelcard_mmc_data);
	/* LCD Controller */
	at91_add_device_lcdc(&panelcard_lcdc_data);
	panelcard_register_backlight();
	/* NOR Flash */
	platform_device_register(&panelcard_flash);
	/* RTC */
	at91_add_device_rtc();

	panelcard_add_w1();

#if defined(CONFIG_INPUT_PORTUXINPUT) || defined(CONFIG_INPUT_PORTUXINPUT_MODULE)
	at91_set_gpio_input(AT91_PIN_PA27, 0);
	at91_set_deglitch(AT91_PIN_PA27, 1);
	platform_device_register(&panelcard_portuxinput_device);
#endif
#if defined(CONFIG_INPUT_AT91_MATRIX) || defined(CONFIG_INPUT_AT91_MATRIX_MODULE)
	platform_device_register(&panelcard_at91_matrix_device);
#endif

	panelcard_add_device_buttons();
}

MACHINE_START(AT91SAM9261EK, "taskit Panel-Card")
	/* Maintainer: taskit GmbH*/
	.phys_io	= AT91_BASE_SYS,
	.io_pg_offst	= (AT91_VA_BASE_SYS >> 18) & 0xfffc,
	.boot_params	= AT91_SDRAM_BASE + 0x100,
	.timer		= &at91sam926x_timer,
	.map_io		= panelcard_map_io,
	.init_irq	= panelcard_init_irq,
	.init_machine	= panelcard_board_init,
MACHINE_END
