/*
 * Portux Panel-PC infrared remote control submodule
 *
 * Copyright (C) 2005-2007 taskit GmbH
 */
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/input.h>
#include <asm/uaccess.h>
#include <linux/portuxinput.h>

static struct input_dev *portuxir;

static irmap *btnmap;

static struct portuxinput_layerstate layer={
	.max=1
};

static short addressfilter=0x100;

static long iochandler(unsigned int cmd, unsigned long arg)
{
	struct irentry entry;
	irmap map;


	switch(cmd) {
	case PORTUXIR_IOC_SETKEY:
	case PORTUXIR_IOC_GETKEY:
	case PORTUXIR_IOC_SETKEYMAP:
	case PORTUXIR_IOC_SETFILTER:
	case PORTUXIR_IOC_SETLAYER:
	case PORTUXIR_IOC_SETMAXLAYER:
		if(!btnmap)
			return -ENXIO;
	}

	if((cmd==PORTUXIR_IOC_SETKEY)||(cmd==PORTUXIR_IOC_GETKEY)) {
		if(copy_from_user(&entry, (void __user *)arg, sizeof(struct irentry)))
			return -EFAULT;
	}
	
	switch(cmd) {
	case PORTUXIR_IOC_SETKEY:
		memcpy(&(*btnmap)[entry.code], &entry.value, sizeof(struct portuxinputevent));
		break;
	case PORTUXIR_IOC_GETKEY:
		memcpy(&entry.value, &((*btnmap)[entry.code]), sizeof(struct portuxinputevent));
		if(copy_to_user((void __user *)arg, &entry, sizeof(struct irentry)))
			return -EFAULT;
		break;
	case PORTUXIR_IOC_SETKEYMAP:
		if(copy_from_user(&map, (void __user *)arg, sizeof(irmap)))
			return -EFAULT;
		memcpy(btnmap, &map, sizeof(irmap));
		break;
	case PORTUXIR_IOC_SETFILTER:
		addressfilter=(short)arg;
		break;
	case PORTUXIR_IOC_SETLAYER:
		if((unsigned char)arg<layer.max)
			layer.cur=(unsigned char)arg;
		else
			return -EINVAL;
		break;
	case PORTUXIR_IOC_SETMAXLAYER:
		if((unsigned char)arg<=PORTUXIR_MAX_LAYER)
			layer.max=(unsigned char)arg;
		else
			return -EINVAL;
		break;
	default:
		return -ENOTTY;
	}
	return 0;
}

static void irhandler(struct ir_t *event)
{
	if(likely(btnmap)) {
		if((addressfilter&0x100)||((addressfilter&0xff)==event->addr)) {
			portuxinput_report_event(portuxir, &(*btnmap)[0][event->data], event->repeat, &layer);
			input_sync(portuxir);
		}
	}
}

static int __init portuxtouch_init(void)
{
	int i;

	if(portuxinput_get_platform_data())
		btnmap=portuxinput_get_platform_data()->irkeymap;
	else
		btnmap=0;

	if(btnmap) {
		portuxir=input_allocate_device();
		portuxir->name="Portux Infrared Controller";
		portuxir->evbit[0]=BIT(EV_KEY)|BIT(EV_REL);
		portuxir->relbit[0]=BIT(REL_X)|BIT(REL_Y);
		for(i=0;i<KEY_MAX;i++)
			set_bit(i, portuxir->keybit);
		input_register_device(portuxir);
		portuxinput_register_ir(&irhandler, &iochandler);
	}
	return 0;
}

static void __exit portuxtouch_exit(void)
{
	portuxinput_deregister_ir(&irhandler);
	input_unregister_device(portuxir);
}

module_init(portuxtouch_init)
module_exit(portuxtouch_exit)

MODULE_AUTHOR("Christian Glindkamp");
MODULE_DESCRIPTION("infrared remote control submodule for portuxinput");
MODULE_LICENSE("GPL v2");

