//-----------------------------------------------------------------------------
//  
// linux/drivers/video/epson/s1d13xxxfb_indirect.h -- Indirect interface 
// function header for Epson S1D13xxx controller frame buffer drivers.
//
// S1D13xxx controllers.
// 
// Copyright(c) Seiko Epson Corporation 2000-2005.
// All rights reserved.
//
// This file is subject to the terms and conditions of the GNU General Public
// License. See the file COPYING in the main directory of this archive for
// more details.
//
//---------------------------------------------------------------------------- 
//---------------------------------------------------------------------------- 
//
// WARNING: This file uses a chip specific include file. The include file 
// contains all hardware information needed to generate proper timings, color
// depth, screen resolution etc.
//
// This driver supports one of the following Epson CRT/TV/LCD controllers:
//
// S1D13717
//
//-----------------------------------------------------------------------------


#ifndef __S1D13XXXFB_INDIRECT_H__
#define __S1D13XXXFB_INDIRECT_H__

#include <linux/module.h>
#include <linux/kernel.h>

#ifdef CONFIG_FBCON_EPSON_INDIRECT

#define REG0022_IMEMADDR1               0x0022
#define REG0024_IMEMADDR2               0x0024
#define REG0026_IAUTOINC                0x0026
#define REG0028_IMEMPORT                0x0028


typedef struct
{
	u16*                pCmd;
	u16*                pData16;
} S1D13XXXFB_IND_STRUCT;

//----------------------------------------------------------------------------
// The following are internal functions/variables to support the indirect interface.
//----------------------------------------------------------------------------

void		s1d13xxxfb_IndirectInit( u32 RegAddr );
void		s1d13xxxfb_IndirectWriteReg( u16 Index, u16 Value );
u16         s1d13xxxfb_IndirectReadReg( u16 Index );
void		s1d13xxxfb_IndirectWriteData( u32 Offset, const void* pData, u32 Count );

//----------------------------------------------------------------------------
// The following are MACROS that need to be defined on a chip by chip basis to 
// work in Indirect mode.  These MACROS were originally defined in s1d13XXX.h,
// where XXX is the chip model.
//   S1D_WRITE_PALETTE
//   S1D_READ_PALETTE
//   S1D_WRITE_PALETTE_LUT2
//   S1D_READ_PALETTE_LUT2
//----------------------------------------------------------------------------

#ifdef CONFIG_FBCON_EPSON_S1D13717
#define S1D_INDIRECT_WRITE_PALETTE(i,r,g,b)  \
{  \
	s1d13xxxfb_IndirectWriteReg( 0x400+(i*sizeof(S1D_VALUE)*2), (S1D_VALUE)(g)<<8 | r ); \
	s1d13xxxfb_IndirectWriteReg( 0x402+(i*sizeof(S1D_VALUE)*2), b ); \
}
#endif


#endif

#endif	//__S1D13XXXFB_INDIRECT_H__
