//-----------------------------------------------------------------------------
//  
// linux/drivers/video/epson/s1d13xxxfb_usb.h -- USB 
// function header for Epson S1D13xxx controller frame buffer drivers.
//
// S1D13xxx controllers.
// 
// Copyright(c) Seiko Epson Corporation 2000-2005.
// All rights reserved.
//
// This file is subject to the terms and conditions of the GNU General Public
// License. See the file COPYING in the main directory of this archive for
// more details.
//
//---------------------------------------------------------------------------- 
//---------------------------------------------------------------------------- 
//
// WARNING: This file is a chip specific include file. The include file 
// contains all hardware information needed to generate proper timings and 
// access registers
//
// This driver supports one of the following Epson CRT/TV/LCD controllers:
//
// S1D13A05
//
//
//-----------------------------------------------------------------------------


#ifndef __S1D13XXXFB_USB_H__
#define __S1D13XXXFB_USB_H__

#ifdef CONFIG_FBCON_EPSON_USB


// 13A05 USB Register Names

#define REG_USB_BASE                        0x4000

#define REG4000_USB_CONTROL                 0x4000
#define REG4002_USB_INT_ENABLE_0            0x4002
#define REG4004_USB_INT_STATUS_0            0x4004
#define REG4006_USB_INT_ENABLE_1            0x4006
#define REG4008_USB_INT_STATUS_1            0x4008
#define REG4010_USB_EP1_INDEX               0x4010
#define REG4012_USB_EP1_RX_MAILBOX_DATA     0x4012
#define REG4018_USB_EP2_INDEX               0x4018
#define REG401A_USB_EP2_TX_MAILBOX_DATA     0x401A
#define REG401C_USB_EP2_INT_POLL_INTERVAL   0x401C
#define REG4020_USB_EP3_RX_FIFO_DATA        0x4020
#define REG4022_USB_EP3_RX_FIFO_COUNT       0x4022
#define REG4024_USB_EP3_RX_FIFO_STATUS      0x4024
#define REG4026_USB_EP3_MAX_PACKET_SIZE     0x4026
#define REG4028_USB_EP4_TX_FIFO_DATA        0x4028
#define REG402A_USB_EP4_TX_FIFO_COUNT       0x402A
#define REG402C_USB_EP4_TX_FIFO_STATUS      0x402C
#define REG402E_USB_EP4_MAX_PACKET_SIZE     0x402E
#define REG4030_USB_REVISION                0x4030
#define REG4032_USB_USB_STATUS              0x4032
#define REG4034_USB_FRAME_COUNTER_MSB       0x4034
#define REG4036_USB_FRAME_COUNTER_LSB       0x4036
#define REG4038_USB_EXT_REGISTER_INDEX      0x4038
#define REG403A_USB_EXT_REGISTER_DATA       0x403A
#define REG4040_USB_USBFC_INPUT_CONTROL     0x4040
#define REG4042_USB_USBFC_OUTPUT_STATUS     0x4042
#define REG4044_USB_PIN_IO_STATUS_DATA      0x4044
#define REG4046_USB_POS_INT_ENABLE          0x4046
#define REG4048_USB_NEG_INT_ENABLE          0x4048
#define REG404A_USB_POS_INT_STATUS          0x404A
#define REG404C_USB_NEG_INT_STATUS          0x404C
#define REG404E_USB_POS_INT_MASK_STATUS     0x404E
#define REG4050_USB_NEG_INT_MASK_STATUS     0x4050
#define REG4052_USB_SOFTWARE_RESET          0x4052
#define REG4054_USB_WAIT_STATE              0x4054

// 13A05 USB Extended Register Names
#define EXT_USB_VENDOR_ID_MSB               0x00
#define EXT_USB_VENDOR_ID_LSB               0x01
#define EXT_USB_PRODUCT_ID_MSB              0x02
#define EXT_USB_PRODUCT_ID_LSB              0x03
#define EXT_USB_RELEASE_NUMBER_MSB          0x04
#define EXT_USB_RELEASE_NUMBER_LSB          0x05
#define EXT_USB_RX_FIFO_FULL_THRESHOLD      0x06
#define EXT_USB_TX_FIFO_EMPTY_THRESHOLD     0x07
#define EXT_USB_USB_CONTROL                 0x08
#define EXT_USB_MAX_POWER                   0x09
#define EXT_USB_PACKET_CONTROL              0x0A
#define EXT_USB_RESERVED                    0x0B
#define EXT_USB_FIFO_CONTROL                0x0C

// Vendor ID's
#define VENDORID_NETCHIP  0x0525

// Product ID's
#define PRODUCTID_INACTIVE  0x1000  // (Product does nothing)
#define PRODUCTID_LOOPBACK  0x2888  // Loopback mode
#define PRODUCTID_COMPORT   0x100c  // Serial COM port
#define PRODUCTID_RESERVED  0x1000

// For testing specify the NetChip Vendor ID and the loopback product ID.
#define VENDORID VENDORID_NETCHIP
#define PRODUCTID PRODUCTID_LOOPBACK

#define VENDORID_MSB (VENDORID>>8)
#define VENDORID_LSB (VENDORID&0x00FF)

#define PRODUCTID_MSB (PRODUCTID>>8)
#define PRODUCTID_LSB (PRODUCTID&0x00FF)

//----------------------------------------------------------------------------
// The following are functions/variables to support the USB interface.
//----------------------------------------------------------------------------
void s1d13xxxfb_USBInit( void );

#endif
#endif	//__S1D13XXXFB_USB_H__
